@@ -10,43 +10,44 @@ import ipaddress
 
 # pylint:disable=R0903
 from abc import ABCMeta
+from typing import Dict, List, Optional, Type, Union
 
-from geoip2.compat import compat_ip_network
 from geoip2.mixins import SimpleEquality
 
 
-class Record(SimpleEquality):
+class Record(SimpleEquality, metaclass=ABCMeta):
     """All records are subclasses of the abstract class ``Record``."""
 
-    __metaclass__ = ABCMeta
-
-    def __repr__(self):
-        args = ', '.join('%s=%r' % x for x in self.__dict__.items())
-        return '{module}.{class_name}({data})'.format(
-            module=self.__module__,
-            class_name=self.__class__.__name__,
-            data=args)
+    def __repr__(self) -> str:
+        args = ", ".join("%s=%r" % x for x in self.__dict__.items())
+        return "{module}.{class_name}({data})".format(
+            module=self.__module__, class_name=self.__class__.__name__, data=args
+        )
 
 
-class PlaceRecord(Record):
+class PlaceRecord(Record, metaclass=ABCMeta):
     """All records with :py:attr:`names` subclass :py:class:`PlaceRecord`."""
 
-    __metaclass__ = ABCMeta
+    names: Dict[str, str]
+    _locales: List[str]
 
-    def __init__(self, locales=None, names=None):
+    def __init__(
+        self,
+        locales: Optional[List[str]] = None,
+        names: Optional[Dict[str, str]] = None,
+    ) -> None:
         if locales is None:
-            locales = ['en']
+            locales = ["en"]
         self._locales = locales
         if names is None:
             names = {}
         self.names = names
 
     @property
-    def name(self):
+    def name(self) -> Optional[str]:
         """Dict with locale codes as keys and localized name as value."""
         # pylint:disable=E1101
-        return next((self.names.get(x)
-                     for x in self._locales if x in self.names), None)
+        return next((self.names.get(x) for x in self._locales if x in self.names), None)
 
 
 class City(PlaceRecord):
@@ -87,15 +88,21 @@ class City(PlaceRecord):
       :type: dict
 
     """
-    def __init__(self,
-                 locales=None,
-                 confidence=None,
-                 geoname_id=None,
-                 names=None,
-                 **_):
+
+    confidence: Optional[int]
+    geoname_id: Optional[int]
+
+    def __init__(
+        self,
+        locales: Optional[List[str]] = None,
+        confidence: Optional[int] = None,
+        geoname_id: Optional[int] = None,
+        names: Optional[Dict[str, str]] = None,
+        **_
+    ) -> None:
         self.confidence = confidence
         self.geoname_id = geoname_id
-        super(City, self).__init__(locales, names)
+        super().__init__(locales, names)
 
 
 class Continent(PlaceRecord):
@@ -135,15 +142,21 @@ class Continent(PlaceRecord):
       :type: dict
 
     """
-    def __init__(self,
-                 locales=None,
-                 code=None,
-                 geoname_id=None,
-                 names=None,
-                 **_):
+
+    code: Optional[str]
+    geoname_id: Optional[int]
+
+    def __init__(
+        self,
+        locales: Optional[List[str]] = None,
+        code: Optional[str] = None,
+        geoname_id: Optional[int] = None,
+        names: Optional[Dict[str, str]] = None,
+        **_
+    ) -> None:
         self.code = code
         self.geoname_id = geoname_id
-        super(Continent, self).__init__(locales, names)
+        super().__init__(locales, names)
 
 
 class Country(PlaceRecord):
@@ -197,19 +210,27 @@ class Country(PlaceRecord):
       :type: dict
 
     """
-    def __init__(self,
-                 locales=None,
-                 confidence=None,
-                 geoname_id=None,
-                 is_in_european_union=False,
-                 iso_code=None,
-                 names=None,
-                 **_):
+
+    confidence: Optional[int]
+    geoname_id: Optional[int]
+    is_in_european_union: bool
+    iso_code: Optional[str]
+
+    def __init__(
+        self,
+        locales: Optional[List[str]] = None,
+        confidence: Optional[int] = None,
+        geoname_id: Optional[int] = None,
+        is_in_european_union: bool = False,
+        iso_code: Optional[str] = None,
+        names: Optional[Dict[str, str]] = None,
+        **_
+    ) -> None:
         self.confidence = confidence
         self.geoname_id = geoname_id
         self.is_in_european_union = is_in_european_union
         self.iso_code = iso_code
-        super(Country, self).__init__(locales, names)
+        super().__init__(locales, names)
 
 
 class RepresentedCountry(Country):
@@ -273,21 +294,25 @@ class RepresentedCountry(Country):
       :type: unicode
 
     """
+
+    type: Optional[str]
+
     def __init__(
-            self,
-            locales=None,
-            confidence=None,
-            geoname_id=None,
-            is_in_european_union=False,
-            iso_code=None,
-            names=None,
-            # pylint:disable=redefined-builtin
-            type=None,
-            **_):
+        self,
+        locales: Optional[List[str]] = None,
+        confidence: Optional[int] = None,
+        geoname_id: Optional[int] = None,
+        is_in_european_union: bool = False,
+        iso_code: Optional[str] = None,
+        names: Optional[Dict[str, str]] = None,
+        # pylint:disable=redefined-builtin
+        type: Optional[str] = None,
+        **_
+    ) -> None:
         self.type = type
-        super(RepresentedCountry,
-              self).__init__(locales, confidence, geoname_id,
-                             is_in_european_union, iso_code, names)
+        super().__init__(
+            locales, confidence, geoname_id, is_in_european_union, iso_code, names
+        )
 
 
 class Location(Record):
@@ -356,25 +381,32 @@ class Location(Record):
       :type: unicode
 
     """
-    def __init__(self,
-                 average_income=None,
-                 accuracy_radius=None,
-                 latitude=None,
-                 longitude=None,
-                 metro_code=None,
-                 population_density=None,
-                 postal_code=None,
-                 postal_confidence=None,
-                 time_zone=None,
-                 **_):
+
+    average_income: Optional[int]
+    accuracy_radius: Optional[int]
+    latitude: Optional[float]
+    longitude: Optional[float]
+    metro_code: Optional[int]
+    population_density: Optional[int]
+    time_zone: Optional[str]
+
+    def __init__(
+        self,
+        average_income: Optional[int] = None,
+        accuracy_radius: Optional[int] = None,
+        latitude: Optional[float] = None,
+        longitude: Optional[float] = None,
+        metro_code: Optional[int] = None,
+        population_density: Optional[int] = None,
+        time_zone: Optional[str] = None,
+        **_
+    ) -> None:
         self.average_income = average_income
         self.accuracy_radius = accuracy_radius
         self.latitude = latitude
         self.longitude = longitude
         self.metro_code = metro_code
         self.population_density = population_density
-        self.postal_code = postal_code
-        self.postal_confidence = postal_confidence
         self.time_zone = time_zone
 
 
@@ -391,7 +423,10 @@ class MaxMind(Record):
       :type: int
 
     """
-    def __init__(self, queries_remaining=None, **_):
+
+    queries_remaining: Optional[int]
+
+    def __init__(self, queries_remaining: Optional[int] = None, **_) -> None:
         self.queries_remaining = queries_remaining
 
 
@@ -422,7 +457,13 @@ class Postal(Record):
       :type: int
 
     """
-    def __init__(self, code=None, confidence=None, **_):
+
+    code: Optional[str]
+    confidence: Optional[int]
+
+    def __init__(
+        self, code: Optional[str] = None, confidence: Optional[int] = None, **_
+    ) -> None:
         self.code = code
         self.confidence = confidence
 
@@ -474,17 +515,24 @@ class Subdivision(PlaceRecord):
       :type: dict
 
     """
-    def __init__(self,
-                 locales=None,
-                 confidence=None,
-                 geoname_id=None,
-                 iso_code=None,
-                 names=None,
-                 **_):
+
+    confidence: Optional[int]
+    geoname_id: Optional[int]
+    iso_code: Optional[str]
+
+    def __init__(
+        self,
+        locales: Optional[List[str]] = None,
+        confidence: Optional[int] = None,
+        geoname_id: Optional[int] = None,
+        iso_code: Optional[str] = None,
+        names: Optional[Dict[str, str]] = None,
+        **_
+    ) -> None:
         self.confidence = confidence
         self.geoname_id = geoname_id
         self.iso_code = iso_code
-        super(Subdivision, self).__init__(locales, names)
+        super().__init__(locales, names)
 
 
 class Subdivisions(tuple):
@@ -498,17 +546,22 @@ class Subdivisions(tuple):
 
     This attribute is returned by ``city``, ``enterprise``, and ``insights``.
     """
-    def __new__(cls, locales, *subdivisions):
-        subdivisions = [Subdivision(locales, **x) for x in subdivisions]
-        obj = super(cls, Subdivisions).__new__(cls, subdivisions)
+
+    def __new__(
+        cls: Type["Subdivisions"], locales: Optional[List[str]], *subdivisions
+    ) -> "Subdivisions":
+        subobjs = tuple(Subdivision(locales, **x) for x in subdivisions)
+        obj = super().__new__(cls, subobjs)  # type: ignore
         return obj
 
-    def __init__(self, locales, *subdivisions):  # pylint:disable=W0613
+    def __init__(
+        self, locales: Optional[List[str]], *subdivisions  # pylint:disable=W0613
+    ) -> None:
         self._locales = locales
-        super(Subdivisions, self).__init__()
+        super().__init__()
 
     @property
-    def most_specific(self):
+    def most_specific(self) -> Subdivision:
         """The most specific (smallest) subdivision available.
 
         If there are no :py:class:`Subdivision` objects for the response,
@@ -734,28 +787,52 @@ class Traits(Record):
       :type: unicode
 
     """
-    def __init__(self,
-                 autonomous_system_number=None,
-                 autonomous_system_organization=None,
-                 connection_type=None,
-                 domain=None,
-                 is_anonymous=False,
-                 is_anonymous_proxy=False,
-                 is_anonymous_vpn=False,
-                 is_hosting_provider=False,
-                 is_legitimate_proxy=False,
-                 is_public_proxy=False,
-                 is_satellite_provider=False,
-                 is_tor_exit_node=False,
-                 isp=None,
-                 ip_address=None,
-                 network=None,
-                 organization=None,
-                 prefix_len=None,
-                 static_ip_score=None,
-                 user_count=None,
-                 user_type=None,
-                 **_):
+
+    autonomous_system_number: Optional[int]
+    autonomous_system_organization: Optional[str]
+    connection_type: Optional[str]
+    domain: Optional[str]
+    is_anonymous: bool
+    is_anonymous_proxy: bool
+    is_anonymous_vpn: bool
+    is_hosting_provider: bool
+    is_legitimate_proxy: bool
+    is_public_proxy: bool
+    is_satellite_provider: bool
+    is_tor_exit_node: bool
+    isp: Optional[str]
+    ip_address: Optional[str]
+    organization: Optional[str]
+    static_ip_score: Optional[float]
+    user_count: Optional[int]
+    user_type: Optional[str]
+    _network: Optional[str]
+    _prefix_len: Optional[int]
+
+    def __init__(
+        self,
+        autonomous_system_number: Optional[int] = None,
+        autonomous_system_organization: Optional[str] = None,
+        connection_type: Optional[str] = None,
+        domain: Optional[str] = None,
+        is_anonymous: bool = False,
+        is_anonymous_proxy: bool = False,
+        is_anonymous_vpn: bool = False,
+        is_hosting_provider: bool = False,
+        is_legitimate_proxy: bool = False,
+        is_public_proxy: bool = False,
+        is_satellite_provider: bool = False,
+        is_tor_exit_node: bool = False,
+        isp: Optional[str] = None,
+        ip_address: Optional[str] = None,
+        network: Optional[str] = None,
+        organization: Optional[str] = None,
+        prefix_len: Optional[int] = None,
+        static_ip_score: Optional[float] = None,
+        user_count: Optional[int] = None,
+        user_type: Optional[str] = None,
+        **_
+    ) -> None:
         self.autonomous_system_number = autonomous_system_number
         self.autonomous_system_organization = autonomous_system_organization
         self.connection_type = connection_type
@@ -780,7 +857,7 @@ class Traits(Record):
     # This code is duplicated for performance reasons
     # pylint: disable=duplicate-code
     @property
-    def network(self):
+    def network(self) -> Optional[Union[ipaddress.IPv4Network, ipaddress.IPv6Network]]:
         """The network for the record"""
         network = self._network
         if isinstance(network, (ipaddress.IPv4Network, ipaddress.IPv6Network)):
@@ -792,6 +869,6 @@ class Traits(Record):
             if ip_address is None or prefix_len is None:
                 return None
             network = "{}/{}".format(ip_address, prefix_len)
-        network = compat_ip_network(network, False)
+        network = ipaddress.ip_network(network, False)
         self._network = network
-        return network
+        return network  # type: ignore
